package processModel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Dimension2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Observable;
import java.util.Observer;

import utilities.AppConstants;
import processView.Externaliser;
import utilities.HandleType;

public class TransitionHandle extends Handle implements Observer{

	private Handle partner = null;
	private Transition parent=null;
	protected int id = -1;
	
	public TransitionHandle(Point p1, Transition parent){
		super(p1, HandleType.TRANSITION_HANDLE);
		this.parent=parent;
	}
	
	public void connect(Handle partner) {
		System.out.println ("ABOUT TO CONNECT: " + this + " TO " + partner);
			if (partner.handleType() == HandleType.STATE_HANDLE){
				this.moveHandleResizingOwner(partner.origin());
				this.partner = partner;
				partner.addObserver(this);
				partner.setConnected(true);
			}
	}
	
	public void addObs(Transition t){
		this.addObserver(t);
	}
	


	public boolean moveHandleResizingOwner(Point newPoint){
		this.origin(newPoint);
		setChanged();
		System.out.println("NOTIFYING >>>");
		notifyObservers(AppConstants.RESIZE_OWNER);
		return true; // change this later to only return true if it was successful
	}
	
	public void disconnect() {
		if (partner != null) partner.deleteObserver(this);
		this.partner = null;
	}


	public Handle getPartner() {
		return partner;
	}
	
	public boolean isConnected() {
		if (partner != null)
			return true;
		else
			return false;
	}
		
	protected Color getHandleColor() {
		if(selected())
			return Color.blue;
		else 
			if (isConnected())
				return Color.green;
			else 
				return Color.red;
	}
	
	public void paint(Graphics2D g2d) {
		Graphics2D g = (Graphics2D) g2d.create(); //create own copy to alter
		Rectangle2D rect = new Rectangle();
		Dimension2D rectDimension = new Dimension (handleSize(), handleSize());
		double topLeftX = origin().getX() - (handleSize()/2);
		double topLeftY = origin().getY() - (handleSize()/2);
		Point topLeftPoint = new Point((int)topLeftX, (int)topLeftY);
		rect.setFrame(topLeftPoint, rectDimension);
		g.setColor(getHandleColor());
		g.fill(rect);
		g.setColor(Color.pink);
	
		/*Point topLeftMaskPoint = new Point (((int) origin().getX() - (handleSize()/2) - MASK_INCREMENT),((int) origin().getY( )- (handleSize()/2) - MASK_INCREMENT));
		Dimension2D rectMaskDimension = new Dimension (handleSize() + (MASK_INCREMENT * 2), handleSize() +  (MASK_INCREMENT * 2));
		Rectangle2D rectMask = new Rectangle();
		rectMask.setFrame(topLeftMaskPoint, rectMaskDimension);
		
		g.draw(rectMask);*/
		g.dispose();
	}

	// The transition handle observes its partner's (StateElement) handle 
	public void update(Observable objStateElement, Object intTypeOfChange) {
		
		if (objStateElement != partner) return;
		int typeOfChange = 0; 
		try {
			typeOfChange = ((Integer)intTypeOfChange);
			
			switch (typeOfChange){
			case AppConstants.MOVE_ELEMENT:
			case AppConstants.RESIZE_ELEMENT:
			case AppConstants.RESIZE_OWNER:
				this.moveHandleResizingOwner(partner.origin());
				break;
			}
		}catch (Exception e){
			System.err.println("Could not move transition handle");
		}	
	}

	public void prepareForDeletion() {
		this.disconnect();
		this.deleteObservers();
	}
	
	public int getID ()
	{
		return id;
	}
	
	public void setID(int id)
	{
		this.id = id;
	}
	
	/*public boolean deleted ()
	{
		return id == -1;
	}*/
	
	public Transition getParent() {
		return parent;
	}

	public String toXML(String indent) {
		String xmlStr = "";
		
		xmlStr += Externaliser.openTag(indent, AppConstants.ELT_POINT + AppConstants.S + AppConstants.ATT_ID 
				+ Externaliser.quoted(String.valueOf(this.getID()))
				+ AppConstants.S + AppConstants.ATT_X + Externaliser.quoted(String.valueOf(this.origin().x)) 
				+ AppConstants.S + AppConstants.ATT_Y + Externaliser.quoted(String.valueOf(this.origin().y)));
		
		if (parent != null) {
				xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_ATTACHED_TRANSITION + AppConstants.S + AppConstants.ATT_REF 
						+ Externaliser.quoted(String.valueOf(parent.getID())));
        }
		
		xmlStr += Externaliser.closeTag(indent, AppConstants.ELT_POINT);
		
		return xmlStr;
	}
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "Point");
		xmlStr += pointASPIRE_XML(indent);
		xmlStr +=  Externaliser.closeInstanceTag(indent);
		return xmlStr;
    }
	
	protected String pointASPIRE_XML(String indent) {
		String xmlStr = "";
		
		return xmlStr;
	}
	
}