package processView;

import java.io.InputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import utilities.AppUtils;

// Maintains connection with ASPIRE server and sends and receives feedback 

public class ASPIREServerConnection{

	protected String SUBMIT_SOLUTION_URL = "process-attempt";
	private URL codeBase;
	private ProcessApplet controller;
	
	public ASPIREServerConnection(ProcessApplet controller) {		
		this.controller = controller;	
		this.codeBase = controller.getCodeBase();
	}
	
	
    public FeedbackMessage submitSolution(String solutionXML, String hintLevel) {
    	URL url = codeBase;    	
        try {
        	System.out.println("Try FeedbackMessage");
        	if (hintLevel != null) {
        		url = new URL(codeBase.toString() + SUBMIT_SOLUTION_URL + "?feedback-level=" + hintLevel);
        	} else {
        		url = new URL(codeBase.toString() + SUBMIT_SOLUTION_URL);
        	}        		
        	System.out.println(url.toString());
            URLConnection connection = url.openConnection();
            connection.setDoOutput(true);
            connection.setDoInput(true);
            PrintWriter out = new PrintWriter(connection.getOutputStream());
            out.print(solutionXML);
            out.close();            
            return processFeedback(connection.getInputStream());                        
        } catch (Exception e) {
        	System.out.println("Catch FeedbackMessage");
            Collection<String> message = new ArrayList<String>();
            message.add(utilities.AppConstants.SYSTEM_ERROR);
            message.add("Unable to get feedback from ASPIRE server: " + url.toString());
            message.add(e.getMessage());
            message.add(utilities.AppConstants.NETWORK_ERROR_MSG);
            message.add(utilities.AppConstants.ERROR_MSG);
            controller.showMessageDialog(utilities.AppConstants.SYSTEM_ERROR_TITLE, AppUtils.assembleMessage(message),
                JOptionPane.ERROR_MESSAGE, controller);
        }
        return null;
    }
	
    public static String FEEDBACK_TAG = "feedback";
    public static String FEEDBACK_TYPE_TAG = "feedback_type";
    public static String NEXT_FEEDBACK_LEVEL_TAG = "next_feedback_level";
    public static String SOLUTION_CORRECT_TAG = "solution_correct_p";
    public static String WHOLE_PAGE_DISPLAYED_TAG = "whole_page_displayed_p";
    public static String LAST_PAGE_TAG = "last_page_p";
    
    public static String ERROR_TAG = "error";
    public static String LOGOUT_TAG = "logout";
	        
    public FeedbackMessage processFeedback(InputStream stream) {
    	
        Document document;
    	
    	ArrayList<String> feedbackItems = new ArrayList<String>();
    	String feedbackType="", nextFeedbackLevel="", solutionCorrect="", wholePageDisplayed="", lastPage="";
    	   
    	
        //factory.setValidating(true);       	
        try {
        	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        	DocumentBuilder builder = factory.newDocumentBuilder();
        	document = builder.parse(stream);
        	//String rootTag = document.getDocumentElement().getTagName();
        	Element rootNode = document.getDocumentElement();
        	System.out.println("Parsed document with elements : " + rootNode.getChildNodes().getLength());
           
        	for (int i = 0; i < rootNode.getChildNodes().getLength(); i++) {
      			Element e = (Element)rootNode.getChildNodes().item(i);
      			String tagName = e.getTagName();
      			String content = e.getTextContent();
    			System.out.println("Child nodees of " + tagName + " - " + e.getChildNodes().getLength());
      			if (tagName.equals(FEEDBACK_TAG)){
      				feedbackItems.add(e.getTextContent());
      			} else if (tagName.equals(FEEDBACK_TYPE_TAG)){
      				feedbackType = e.getTextContent();
      			} else if (tagName.equals(NEXT_FEEDBACK_LEVEL_TAG)){
      				nextFeedbackLevel = e.getTextContent();
      			} else if (tagName.equals(SOLUTION_CORRECT_TAG)){
      				solutionCorrect = e.getTextContent();
      			} else if (tagName.equals(WHOLE_PAGE_DISPLAYED_TAG)){
      				wholePageDisplayed = e.getTextContent();
      			} else if (tagName.equals(LAST_PAGE_TAG)){
      				lastPage = e.getTextContent();
      			}
        	}
           
           /* 
            * TODO: Need to check for ERROR And LOGOUT tags
            * 
           else if (rootTag.equals(ERROR_TAG)){
        	   
           } else if (rootTag.equals(LOGOUT_TAG)){
        	              	
           } */             
           System.out.println("There are : " + feedbackItems.size()+ " feedback items");
           System.out.println("Feedback type is  : " + feedbackType );
           
           String feedbackText;
           if ("list".equals(feedbackType)) {
        	   Iterator<String> feedbackIt = feedbackItems.iterator();
        	   feedbackText = "<ol>";;           
        	   while (feedbackIt.hasNext()) {        	   
        		   feedbackText+="\n  <li>"+ feedbackIt.next() + "</li>";
        	   }
        	   feedbackText+="</ol>";
           } else {
        	   if(feedbackItems.size() >0){
        		   String feedbackLines[] = feedbackItems.get(0).split("\n");
        		   feedbackText = "";
	        	   for (int i = 0 ; i < feedbackLines.length; i++ ) {
	        		   feedbackText += "<p>" + feedbackLines[i] + "</p>";  
	        	   }
        	   }else feedbackText = "";
           }
           System.out.println("FeedbackText : " + feedbackText);
           		
        
           FeedbackMessage msg = new FeedbackMessage();
           msg.setFeedbackText(feedbackText);
           msg.setNextFeedbackLevel(nextFeedbackLevel);
           msg.setSolutionCorrect(solutionCorrect);
           
           return msg;
           
        } catch (Exception e) {
        	e.printStackTrace();
        }
        
        return null;
                
    }

    
        
}
