package processView;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.FocusTraversalPolicy;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import processModel.Transition;
import processModel.Process;
import utilities.AppConstants;
import utilities.AppUtils;
import utilities.SpringUtilities;
import javax.swing.SpringLayout;

import controls.UnitComboBox;

public class TransitionDialogMenu extends DialogMenu {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3927687138681917125L;
	private Transition transition = null;
	private String heat = "";
	private String work = "";
	private String heatUnit = "";
	private String workUnit = "";
	private String deltaUnit = "";
	private JTextField txtWork;
	private JTextField txtHeat;
	private JTextField txtDeltaU;
	private UnitComboBox cmbWorkUnit;
	private UnitComboBox cmbHeatUnit;
	private UnitComboBox cmbDeltaUUnit;
	private JCheckBox chkWorkUnknown;
	private JCheckBox chkHeatUnknown;
	private JCheckBox chkDeltaUnknown;

	public TransitionDialogMenu(Frame owner, String title,
			Transition transition, Process process) {
		super(owner, title, process);
		this.transition = transition;
		this.label(transition.label());
		this.heat(transition.heat());
		this.work(transition.work());
		this.delta(transition.delta());
		this.heatUnit(transition.getHeatUnits());
		this.workUnit(transition.getWorkUnits());
		this.deltaUnit(transition.getDeltaUnits());
		this.setSize(200, 600);
		createGUI();
		setFocusTraversalPolicy(new TransitionDialogFocusTraversalPolicy());
	}

	public void heat(String heat) {
		if (heat == null)
			this.heat = "";
		else
			this.heat = heat.trim();
	}

	public void work(String work) {
		if (work == null)
			this.work = "";
		else
			this.work = work.trim();
	}

	public void heatUnit(String heatUnit) {
		if (heatUnit == null)
			this.heatUnit = "";
		else
			this.heatUnit = heatUnit.trim();
	}

	public void workUnit(String workUnit) {
		if (workUnit == null)
			this.workUnit = "";
		else
			this.workUnit = workUnit.trim();
	}

	public void deltaUnit(String deltaUnit) {
		if (deltaUnit == null)
			this.deltaUnit = "";
		else
			this.deltaUnit = deltaUnit.trim();
	}

	@Override
	public void label(String label) {
	}

	@Override
	protected void createGUI() {

		// The Panels
		JPanel contentPane = new JPanel();
		AppUtils.restrictSize(contentPane, new Dimension(400, 220));
		JPanel pnlMain = new JPanel(new SpringLayout());

		JLabel lbl1 = new JLabel(" Property");
		lbl1.setFont(AppConstants.arialBold);
		pnlMain.add(lbl1);
		JLabel lbl2 = new JLabel("Value");
		lbl2.setFont(AppConstants.arialBold);
		pnlMain.add(lbl2);
		JLabel lbl4 = new JLabel("Units");
		lbl4.setFont(AppConstants.arialBold);
		pnlMain.add(lbl4);
		JLabel lbl3 = new JLabel("Unknown");
		lbl3.setFont(AppConstants.arialBold);
		pnlMain.add(lbl3);

		// The Components
		Font font = new Font("Arial", Font.PLAIN, 12);

		JLabel lblWork = new JLabel(" Work ");
		pnlMain.add(lblWork);
		txtWork = new JTextField(work, 10);
		pnlMain.add(txtWork);
		cmbWorkUnit = new UnitComboBox(process);
		cmbWorkUnit.setSelectedItemByString(workUnit);
		pnlMain.add(cmbWorkUnit);
		lblWork.setFont(font);

		chkWorkUnknown = new JCheckBox();
		if (transition.workUnknown()) {
			chkWorkUnknown.setSelected(true);
			txtWork.setEnabled(false);
			cmbWorkUnit.setEnabled(false);
			txtWork.setText("");
			cmbWorkUnit.setSelectedIndex(0);

		}

		chkWorkUnknown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtWork.setEnabled(!txtWork.isEnabled());
				cmbWorkUnit.setEnabled(!cmbWorkUnit.isEnabled());
				if (!txtWork.isEnabled()) {
					txtWork.setText("");
					cmbWorkUnit.setSelectedIndex(0);
				}
			}
		});
		pnlMain.add(chkWorkUnknown);
		txtWork.requestFocusInWindow();

		JLabel lblHeat = new JLabel(" Heat ");
		pnlMain.add(lblHeat);
		txtHeat = new JTextField(heat, 10);
		pnlMain.add(txtHeat);
		cmbHeatUnit = new UnitComboBox(process);
		cmbHeatUnit.setSelectedItemByString(heatUnit);
		pnlMain.add(cmbHeatUnit);
		lblHeat.setFont(font);

		chkHeatUnknown = new JCheckBox();
		if (transition.heatUnknown()) {
			chkHeatUnknown.setSelected(true);
			txtHeat.setEnabled(false);
			cmbHeatUnit.setEnabled(false);
			txtHeat.setText("");
			cmbHeatUnit.setSelectedIndex(0);

		}

		chkHeatUnknown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtHeat.setEnabled(!txtHeat.isEnabled());
				cmbHeatUnit.setEnabled(!cmbHeatUnit.isEnabled());
				if (!txtHeat.isEnabled()) {
					txtHeat.setText("");
					cmbHeatUnit.setSelectedIndex(0);
				}
			}
		});
		pnlMain.add(chkHeatUnknown);

		JLabel lblDelta = new JLabel(" Change in Energy ");
		pnlMain.add(lblDelta);
		txtDeltaU = new JTextField(delta(), 10);
		pnlMain.add(txtDeltaU);
		cmbDeltaUUnit = new UnitComboBox(process);
		cmbDeltaUUnit.setSelectedItemByString(deltaUnit);
		pnlMain.add(cmbDeltaUUnit);
		lblDelta.setFont(font);

		chkDeltaUnknown = new JCheckBox();
		if (transition.deltaUnknown()) {
			chkDeltaUnknown.setSelected(true);
			txtDeltaU.setEnabled(false);
			cmbDeltaUUnit.setEnabled(false);
			txtDeltaU.setText("");
			cmbDeltaUUnit.setSelectedIndex(0);

		}

		chkDeltaUnknown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtDeltaU.setEnabled(!txtDeltaU.isEnabled());
				cmbDeltaUUnit.setEnabled(!cmbDeltaUUnit.isEnabled());
				if (!txtDeltaU.isEnabled()) {
					txtDeltaU.setText("");
					cmbDeltaUUnit.setSelectedIndex(0);
				}
			}
		});
		pnlMain.add(chkDeltaUnknown);

		SpringUtilities.makeCompactGrid(pnlMain, 4, 4, 3, 3, 7, 7);
		pnlMain.setAlignmentX(Component.CENTER_ALIGNMENT);

		contentPane.add(pnlMain);
		contentPane.add(createButtons(), BorderLayout.SOUTH);

		this.add(contentPane);

		addWindowListener(new WindowAdapter() {
			// capture the window closing event i.e clicking 'X'
			public void windowClosing(WindowEvent evt) {
				if (process.newTransition != null) {
					process.removeTransition(transition);
					dispose();
				} else
					dispose();
			}
		});
	}

	protected JComponent createButtons() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.TRAILING));

		JButton btnCancel = new JButton("Cancel");

		// final properties - for use in the ActionListener
		final DialogMenu dialog = this;

		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (process.newTransition != null) {
					process.removeTransition(transition);
					dialog.setVisible(false);
					dialog.dispose();
				} else {
					dialog.setVisible(false);
					dialog.dispose();
				}
			}
		});

		panel.add(btnCancel);

		JButton btnOK = new JButton("OK");
		btnOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				boolean closeDialog = true;

				String workTxt = txtWork.getText().trim();
				String heatTxt = txtHeat.getText().trim();
				String deltaTxt = txtDeltaU.getText().trim();

				if (chkWorkUnknown.isSelected()) {
					transition.setWorkUnknown();
				} else {
					if (!workTxt.equals(work) || (workTxt.equals("") && transition.workUnknown())) {
						transition.work(workTxt);
					}
				}

				if (chkHeatUnknown.isSelected()) {
					transition.setHeatUnknown();
				} else {
					if (!heatTxt.equals(heat) || (heatTxt.equals("") && transition.heatUnknown())) {
						transition.heat(heatTxt);
					}
				}

				if (chkDeltaUnknown.isSelected()) {
					transition.setDeltaUnknown();
				} else {
					if (!deltaTxt.equals(delta()) || (deltaTxt.equals("") && transition.deltaUnknown())) {
						transition.delta(deltaTxt);
					}
				}

				if (!cmbHeatUnit.equals(heatUnit)) {
					transition.heatUnit(cmbHeatUnit.getSelectedItemString());
				}
				if (!cmbWorkUnit.equals(workUnit)) {
					transition.workUnit(cmbWorkUnit.getSelectedItemString());
				}
				if (!cmbDeltaUUnit.equals(deltaUnit)) {
					transition.deltaUnit(cmbDeltaUUnit.getSelectedItemString());
				}
				
				if (closeDialog) {
					dialog.setVisible(false);
					dialog.dispose();
				}
			}
		});

		panel.add(btnOK);

		// Match the SpringLayout's gap, subtracting 5 to make
		// up for the default gap FlowLayout provides.
		panel.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 5));
		return panel;
	}
	
	class TransitionDialogFocusTraversalPolicy extends FocusTraversalPolicy
	{
		private List<Component> componentsList;
		
		public TransitionDialogFocusTraversalPolicy()
		{
			componentsList = new ArrayList<Component>();
			componentsList.add(txtWork);
			componentsList.add(cmbWorkUnit);
			componentsList.add(txtHeat);
			componentsList.add(cmbHeatUnit);
			componentsList.add(txtDeltaU);
			componentsList.add(cmbDeltaUUnit);
		}
		
		public Component getComponentAfter(Container container, Component component) {
			int currentIndex = componentsList.indexOf(component);
			for (int i = 1; i < componentsList.size(); i++) {
				int index = (currentIndex+i+componentsList.size()) % componentsList.size();
				Component nextComponent = componentsList.get(index);
				if (nextComponent.isEnabled()) {
					return nextComponent;
				}
			}
			return null;
		}

		public Component getComponentBefore(Container container, Component component) {
			int currentIndex = componentsList.indexOf(component);
			for (int i = 1; i < componentsList.size(); i++) {
				int index = (currentIndex-i+componentsList.size()) % componentsList.size();
				Component prevComponent = componentsList.get(index);
				if (prevComponent.isEnabled()) {
					return prevComponent;
				}
			}
			return null;
		}

		public Component getDefaultComponent(Container container) {
			for (Component comp : componentsList) {
				if (comp.isEnabled()) {
					return comp;
				}
			}
			return componentsList.get(0);
		}

		public Component getFirstComponent(Container container) {
			return componentsList.get(0);
		}

		public Component getLastComponent(Container container) {
			return componentsList.get(componentsList.size()-1);
		}
	}
	
}